﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TfsSdkDemo;

namespace TfsSdkDemoApp
{

    /* This is the driver app for TFS SDK.
     */

    public partial class ftest : Form
    {
        TfsDemo tfs;                // the main object.
        TfsDemo.CallBack callBack;  // Callback fuction that will be called when a detection is detected.
        static string path = "tfsCallBackDir";
        static DateTime startTime;
        uint counter = 0;

        public ftest()
        {
            InitializeComponent();
        }
   
        private void btnStart_Click(object sender, EventArgs e)
        {

            /// uncomment this if using "callBackFileCreator"

            /* System.IO.Directory.CreateDirectory(path);
            startTime = DateTime.Now; */
            counter = 0;

            callBack = new TfsDemo.CallBack(callBackMethod);            // setting up the callback function.
            tfs = TfsDemo.getInstance(callBack, txtPathToMonitor.Text, false, "*.*", true, 2);   // this gets you an instance of TFS.
            tfs.addNotification(TfsDemo.DetectionType.Created);         // registers for "Create" detections.
            tfs.addNotification(TfsDemo.DetectionType.Renamed);         // registers for "Rename" detections.
            tfs.startMonitoring();                                      // starts the monitoring for detections.
            list.Items.Add("Started. " + tfs.GetHashCode());
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            tfs.stopMonitoring();                                       // stops monitoring.
            list.Items.Add("Stopped.");
        }

        /// <summary>
        /// This is the callback function, the soul of your app.
        /// this function gets called when any detection is detected.
        /// you add most of your functional logic here.
        /// </summary>
        /// <param name="vars"></param>
        private void callBackMethod(TfsDemo.tfsVars vars)
        {
            //FileInfo info = new FileInfo(vars.Path); // this will get you most of the details of the file [NOT valid in case of delete].
            this.Invoke((MethodInvoker)delegate
            {
                // details about the detection listed on the list.
                if (vars.detectionType == TfsDemo.DetectionType.Renamed)
                    list.Items.Add(vars.dateTime + " - " + vars.oldFileName + " - was renamed to: " + vars.fileName);
                else
                {
                    list.Items.Add(vars.dateTime + " - " + vars.fileName + " was " + vars.detectionType.ToString());
                }

                // updates the display counter.
                lbl.Text = ++counter + " items.";
            });

        }

        /// <summary>
        /// This code assumed that the created files were about 1KB in length. For larger files, it is a different story.
        /// As the file size is quite small, we can almost always be sure that file copy process completed 
        /// before TFS post-processes the copied file. If the file is larger, then TFS may report the detection
        /// before copying was completed fully and this code will copy . So in that case, you would need to 
        /// know when exactly the file copying was completed and then do the post-processing. It’s a bit 
        /// complex, but can be done.
        /// </summary>
        /// <param name="vars"></param>
        private void callBackFileCreator(TfsDemo.tfsVars vars)
        {
            using (StreamWriter sw = new StreamWriter(path + "\\" + vars.fileName, false))
            {
                using (StreamReader sr = new StreamReader("U:\\tfs\\test\\" + vars.fileName))
                {
                    sw.WriteLine(sr.ReadToEnd());
                }
            }
            this.Invoke((MethodInvoker)delegate
            {
                lbl.Text = ++counter + " items.";
            });
        }
    }
}
